# coding: utf-8
"""
レンダリング画像にコサイン4乗則による周辺光量落ち効果をかけます
"""

__author__ = "CASPAR003 <caspar003@caspar003.info>"
__date__ = "2014-07-19/2014-07-19"
__version__ = "1.0"

def ca3_pe_falloff(img, f):
    """
    画像にコサイン4乗則による光量落ちフィルタをかけます
    @param img フィルタをかけたい画像（Shade imageオブジェクト）
    @param f   焦点距離 [mm]
    """
    import math
    import numpy as np
    
    # 画像中心から角までの距離 [mm]
    rmm = math.hypot(24.0, 36.0)/2.0
    
    # 作業用配列の確保
    matr = np.zeros(img.size)
    matg = np.zeros(img.size)
    matb = np.zeros(img.size)
    matf = np.zeros(img.size)
    
    # Shade imageをnumpy arrayに代入
    for ix in range(img.size[0]):
        for iy in range(img.size[1]):
            matr[ix, iy], matg[ix, iy], matb[ix, iy] = img.get_pixel(ix, iy)
    
    # 光量落ちフィルタ
    cx = img.size[0]/2    # 画像の中心点座標
    cy = img.size[1]/2    # 
    rpx = math.hypot(cx, cy)    # 画像中心から角までの距離 [px]
    for ix in range(img.size[0]):
        for iy in range(img.size[1]):
            d = math.hypot(ix + 0.5 - cx, iy + 0.5 - cy)    # 画像中心からピクセル中心までの距離 [mm]
            d /= rpx
            d *= rmm    # 画像中心からピクセル中心までの距離 [mm]
            t = math.atan(d/f)    # 光軸からのずれ角 [rad]
            matf[ix, iy] = math.cos(t)**4
    
    # フィルタの適用
    matr = matf * matr
    matg = matf * matg
    matb = matf * matb
    
    # numpy arrayをShade imageに戻す
    for ix in range(img.size[0]):
        for iy in range(img.size[1]):
            img.set_pixel(ix, iy, (matr[ix, iy], matg[ix, iy], matb[ix, iy]))
    return



if (__name__ == "__main__"):
    img = xshade.scene().rendering.image
    f = xshade.scene().camera.zoom
    if (img is not None):
        ca3_pe_falloff(img, f)
        xshade.scene().rendering.update_image()
