# coding:utf-8
"""
レンダリングウィンドウにある画像をサムネイルとして保存します．
widgetから呼び出す用のスクリプトで読み込み時に[[CURRENT_DIR]]をカレントディレクトリのパスに置換して実行します．
"""

__author__ = "CASPAR003 <caspar003@caspar003.info>"
__date__ = "2014-07-26/2014-07-28"
__version__ = "1.0"

def createBeforeImg():
    """
    レンダリングウィンドウの内容をサムネイルとして保存します．
    @return "{0:d}.format(int)" レンダリング画像のハッシュ値．画像が存在しなかったときは"0"を返します．
    """
    import urllib
    import os
    
    CurrentDir = "[[CURRENT_DIR]]"
    OutFileName = "ri_before.png"
    TmbFileName = "ri_thumb_before.png"
    ThumbneilSize = 280
    
    # レンダリング画像を取得
    try:
        img = xshade.scene().rendering.image
    except:
        img = None
    
    if (img is None):
        # 画像が存在しなければ作る
        img = xshade.create_image((ThumbneilSize, ThumbneilSize), 128)
        result = "{0:d}".format(0)
    else:
        # 画像が存在すればコピーをとる
        img = img.duplicate()
        result = "{0:d}".format(img.__hash__())
    
    # 既にファイルが存在しているときは削除する
    outFilePath = urllib.unquote(CurrentDir) + "images/" + OutFileName
    if os.path.exists(outFilePath): os.remove(outFilePath)
    tmbFilePath = urllib.unquote(CurrentDir) + "images/" + TmbFileName
    if os.path.exists(tmbFilePath): os.remove(tmbFilePath)
    
    # 実寸の画像を保存
    img.save(outFilePath)
    
    # サムネイルサイズに縮小
    if (max(img.size) > ThumbneilSize):
        scale = float(ThumbneilSize) / float(max(img.size))
        img = img.duplicate((int(scale * img.size[0]), int(scale * img.size[1])), True, img.pixel_depth)
    
    img.save(tmbFilePath)
    
    return result

if (__name__ == "__main__"): result = createBeforeImg()
